@@ -7,9 +7,6 @@
 NTL_OPEN_NNS
 
 struct IntFactor {
-   IntFactor() { }
-   ~IntFactor() { }
-
    long q;
    long a;
    long val;
@@ -17,7 +14,7 @@ struct IntFactor {
 };
 
 
-NTL_vector_decl(IntFactor,vec_IntFactor)
+typedef Vec<IntFactor> vec_IntFactor;
 typedef vec_IntFactor FacVec;
 
 void FactorInt(FacVec& fvec, long n);
