@@ -7,29 +7,91 @@
 
 NTL_OPEN_NNS
 
+
+
+
+// Context, Bak, and Push types, just for consistency.
+// They don't do anything
+
+class GF2Context {
+public:
+GF2Context() {}
+explicit GF2Context(long p) {  if (p != 2) LogicError("GF2Context with p != 2"); }
+void save() {}
+void restore() const {}
+};
+
+class GF2Bak {
+public:
+void save();
+void restore();
+
+
+private:
+GF2Bak(const GF2Bak&); // disabled
+void operator=(const GF2Bak&); // disabled
+
+
+};
+
+class GF2Push {
+
+GF2Push(const GF2Push&); // disabled
+void operator=(const GF2Push&); // disabled
+
+public:
+GF2Push() { }
+explicit GF2Push(const GF2Context& context) { }
+explicit GF2Push(long p) { if (p != 2) LogicError("GF2Push with p != 2"); }
+
+
+};
+
+class GF2X; // forward declaration
+
+class ref_GF2; // forward declaration
+
 class GF2 {
 public:
+typedef long rep_type;
+typedef GF2Context context_type;
+typedef GF2Bak bak_type;
+typedef GF2Push push_type;
+typedef GF2X poly_type;
+
 
 unsigned long _GF2__rep;
 
 
 GF2() : _GF2__rep(0) { }
-GF2(const GF2& a) : _GF2__rep(a._GF2__rep) { }
+
+explicit GF2(long a) : _GF2__rep(0) { *this = a; }
 
 GF2(INIT_VAL_TYPE, long a) : _GF2__rep(a & 1) { }
 GF2(INIT_LOOP_HOLE_TYPE, unsigned long a) : _GF2__rep(a) { }
 
-~GF2() { }
+inline GF2(const ref_GF2&);
+
 
-GF2& operator=(const GF2& a) { _GF2__rep = a._GF2__rep; return *this; }
 GF2& operator=(long a) { _GF2__rep = a & 1; return *this; }
 
 static long modulus() { return 2; }
 static GF2 zero() { return GF2(); }
 
+// for consistency
+GF2(INIT_NO_ALLOC_TYPE) : _GF2__rep(0) { } 
+GF2(INIT_ALLOC_TYPE) : _GF2__rep(0) { } 
+void allocate() { }
+
+void swap(GF2& x) { GF2 t; t = *this; *this = x; x = t; }
+
+
 };
 
 
+NTL_DECLARE_RELOCATABLE((GF2*))
+
+
 
 class ref_GF2 {
 public:
@@ -38,19 +100,13 @@ unsigned long *_ref_GF2__ptr;
 long _ref_GF2__pos;
 
 ref_GF2() : _ref_GF2__ptr(0), _ref_GF2__pos(0) { }
-ref_GF2(const ref_GF2& a) : 
-   _ref_GF2__ptr(a._ref_GF2__ptr), _ref_GF2__pos(a._ref_GF2__pos) { }
+
 ref_GF2(GF2& a) :
    _ref_GF2__ptr(&a._GF2__rep),  _ref_GF2__pos(0) { }
+
 ref_GF2(INIT_LOOP_HOLE_TYPE, unsigned long *ptr, long pos) :
    _ref_GF2__ptr(ptr), _ref_GF2__pos(pos) { }
 
-operator const GF2() const 
-{
-   return GF2(INIT_LOOP_HOLE, (*_ref_GF2__ptr >> _ref_GF2__pos) & 1);
-}
-
-~ref_GF2() { }
 
 ref_GF2 operator=(const ref_GF2& a)
 {
@@ -80,10 +136,21 @@ ref_GF2 operator=(long a)
    return *this;
 }
 
+void swap(ref_GF2 x) { GF2 t; t = *this; *this = x; x = t; }
+
 
 };
 
 
+// I changed the conversion from a ref_GF2 operator
+// to a GF2 constructor, because clang was giving me errors
+// Note that gcc, icc, and MS compilers were all OK with
+// the old code
+
+inline
+GF2::GF2(const ref_GF2& other) :
+_GF2__rep((*other._ref_GF2__ptr >> other._ref_GF2__pos) & 1)
+{ }
 
 
 // functions
@@ -152,7 +219,7 @@ inline GF2 operator*(long a, GF2 b)
 
 inline GF2 operator/(GF2 a, GF2 b)
 {
-   if (IsZero(b)) Error("GF2: division by zero");
+   if (IsZero(b)) ArithmeticError("GF2: division by zero");
    return a;
 }
 
@@ -241,8 +308,7 @@ inline void clear(GF2& x) { x = 0; }
 
 inline void set(GF2& x) { x = 1; }
 
-inline void swap(GF2& x, GF2& y)
-   { GF2 t; t = x; x = y; y = t; }
+inline void swap(GF2& x, GF2& y) { x.swap(y); }
 
 inline void add(GF2& x, GF2 a, GF2 b)
    { x = a + b; }
@@ -349,8 +415,7 @@ inline void clear(ref_GF2 x) { x = 0; }
 
 inline void set(ref_GF2 x) { x = 1; }
 
-inline void swap(ref_GF2 x, ref_GF2 y)
-   { GF2 t; t = x; x = y; y = t; }
+inline void swap(ref_GF2 x, ref_GF2 y) { x.swap(y); }
 
 inline void add(ref_GF2 x, GF2 a, GF2 b)
    { x = a + b; }
@@ -432,6 +497,8 @@ inline void conv(ref_GF2 x, GF2 a) { x = a; }
 /* ------------------------------------- */
 
 
+
+
 // Finally, we declare an specialization Vec<GF2>:
 
 template<> class Vec<GF2>;
