@@ -4,14 +4,55 @@
 
 #include <NTL/config.h>
 #include <NTL/mach_desc.h>
-#include <NTL/HAVE_LL_TYPE.h>
-#include <NTL/HAVE_BUILTIN_CLZL.h>
-#include <NTL/HAVE_AVX.h>
-#include <NTL/HAVE_FMA.h>
+
+#include <NTL/ALL_FEATURES.h>
+
+#include <NTL/PackageInfo.h>
+
+#if (defined(__GNUC__) && (defined(__i386__) || defined(__x86_64__)))
+#define NTL_GNUC_INTEL
+#endif
+
+#if (!defined(NTL_HAVE_LL_TYPE) && defined(NTL_WINPACK) &&  (defined(_MSC_VER) || defined(NTL_GNUC_INTEL)))
+// for the windows distribution, 
+//   we assume LL_TYPE works for MSVC++ (which is true for both x86 and ARM)
+//   and for GNUC/Intel platforms (e.g., Code Blocks)
+#define NTL_HAVE_LL_TYPE
+#endif
+
+// Define the working C++ standard.
+// Both NTL_STD_CXX14 and NTL_STD_CXX11, and we take the highest one
+
+#if defined(NTL_STD_CXX14)
+#define NTL_CXX_STANDARD (2014)
+#elif defined(NTL_STD_CXX11)
+#define NTL_CXX_STANDARD (2011)
+#else
+#define NTL_CXX_STANDARD (1998)
+#endif
+
+// define some macros regarding noexcept declarations
+
+#if (NTL_CXX_STANDARD >= 2011)
+
+#define NTL_NOEXCEPT noexcept
+
+#ifdef NTL_EXCEPTIONS
+#define NTL_FAKE_NOEXCEPT
+#else
+#define NTL_FAKE_NOEXCEPT noexcept
+#endif
+
+#else
+
+#define NTL_NOEXCEPT 
+#define NTL_FAKE_NOEXCEPT
+
+#endif
 
 
 /*
- * Resolve double-word integer types.
+ * Resolve double-word integer type.
  *
  * Unfortunately, there is no "standard" way to do this.
  * On 32-bit machines, 'long long' usually works (but not
@@ -20,37 +61,12 @@
  * non-standard double-word type.  
  *
  * Note that C99 creates a standard header <stdint.h>,
- * but it is not clear how widely this is implemented yet,
- * and for example, GCC does not provide a type int128_t 
+ * but it is not clear how widely this is implemented,
+ * and for example, older versions of GCC does not provide a type int128_t 
  * in <stdint.h> on 64-bit machines.
  */
 
 
-#if (defined(NTL_LONG_LONG_TYPE))
-
-#define NTL_LL_TYPE NTL_LONG_LONG_TYPE
-
-#elif (NTL_BITS_PER_LONG == 64 && defined(__GNUC__))
-
-#define NTL_LL_TYPE __int128_t
-
-#elif (NTL_BITS_PER_LONG == 32 && (defined(_MSC_VER) || defined(__BORLANDC__)))
-
-#define NTL_LL_TYPE __int64
-
-#elif (NTL_BITS_PER_LONG == 64 && (defined(_MSC_VER) || defined(__BORLANDC__)))
-
-#define NTL_LL_TYPE __int128
-
-#endif
-
-#if (!defined(NTL_LL_TYPE))
-
-#define NTL_LL_TYPE long long
-
-#endif
-
-
 
 #if (defined(NTL_UNSIGNED_LONG_LONG_TYPE))
 
@@ -58,7 +74,7 @@
 
 #elif (NTL_BITS_PER_LONG == 64 && defined(__GNUC__))
 
-#define NTL_ULL_TYPE __uint128_t
+#define NTL_ULL_TYPE __uint128_t 
 
 #elif (NTL_BITS_PER_LONG == 32 && (defined(_MSC_VER) || defined(__BORLANDC__)))
 
@@ -79,17 +95,14 @@
 
 #ifdef NTL_HAVE_LL_TYPE
 
-typedef NTL_LL_TYPE _ntl_longlong;
 typedef NTL_ULL_TYPE _ntl_ulonglong;
 // typenames are more convenient than macros
 
 #else
 
-#undef NTL_LL_TYPE
 #undef NTL_ULL_TYPE
 // prevent any use of these macros
 
-class _ntl_longlong { private: _ntl_longlong() { } };
 class _ntl_ulonglong { private: _ntl_ulonglong() { } };
 // cannot create variables of these types
 
@@ -293,7 +306,7 @@ extern unsigned long exception_counter;
 
 #define NTL_ULONG_TO_LONG(a) \
    ((((unsigned long) a) >> (NTL_BITS_PER_LONG-1)) ? \
-    (((long) (((unsigned long) a) - ((unsigned long) NTL_MIN_LONG))) + \
+    (((long) (((unsigned long) a) ^ ((unsigned long) NTL_MIN_LONG))) ^ \
        NTL_MIN_LONG) : \
     ((long) a))
 
@@ -308,7 +321,7 @@ extern unsigned long exception_counter;
 
 #define NTL_UINT_TO_INT(a) \
    ((((unsigned int) a) >> (NTL_BITS_PER_INT-1)) ? \
-    (((int) (((unsigned int) a) - ((unsigned int) NTL_MIN_INT))) + \
+    (((int) (((unsigned int) a) ^ ((unsigned int) NTL_MIN_INT))) ^ \
        NTL_MIN_INT) : \
     ((int) a))
 
@@ -347,6 +360,9 @@ extern unsigned long exception_counter;
 
 
 
+double _ntl_GetWallTime();
+
+
 long _ntl_IsFinite(double *p);
 /* This forces a double into memory, and tests if it is "normal";
    that means, not NaN, not +/- infinity, not denormalized, etc.
@@ -358,6 +374,7 @@ void _ntl_ForceToMem(double *p);
 /* This is do-nothing routine that has the effect of forcing
    a double into memory (see comment above). */
 
+
 double _ntl_ldexp(double x, long e);
 
 
@@ -394,6 +411,38 @@ void _ntl_swap(T*& a, T*& b)
  */
 
 
+// The following do for "move" what the above does for swap
+
+#define NTL_DEFINE_SCALAR_MOVE(T)\
+inline T _ntl_scalar_move(T& a)\
+{\
+   T t = a; a = 0; return t;\
+}
+
+NTL_DEFINE_SCALAR_MOVE(long)
+NTL_DEFINE_SCALAR_MOVE(int)
+NTL_DEFINE_SCALAR_MOVE(short)
+NTL_DEFINE_SCALAR_MOVE(char)
+
+NTL_DEFINE_SCALAR_MOVE(unsigned long)
+NTL_DEFINE_SCALAR_MOVE(unsigned int)
+NTL_DEFINE_SCALAR_MOVE(unsigned short)
+NTL_DEFINE_SCALAR_MOVE(unsigned char)
+
+NTL_DEFINE_SCALAR_MOVE(double)
+NTL_DEFINE_SCALAR_MOVE(float)
+
+   
+template<class T>
+T* _ntl_scalar_move(T*& a)
+{
+   T *t = a; a = 0; return t;
+}
+
+
+
+
+
 // The following routine increments a pointer so that
 // it is properly aligned.  
 // It is assumed that align > 0.
@@ -405,7 +454,7 @@ void _ntl_swap(T*& a, T*& b)
 #define NTL_UPTRINT_T unsigned long long
 // DIRT: this should really be std::uintptr_t, defined
 // in <cstdint>; however, that header is not widely available,
-// and even if it were, std::uintpre_t is not guaranteed
+// and even if it were, std::uintptr_t is not guaranteed
 // to be defined.  Of course, unsigned long long may not
 // be defined in pre-C++11.  
 
@@ -416,32 +465,24 @@ void _ntl_swap(T*& a, T*& b)
 #endif
 
 
-#if (!defined(__GNUC__) || !defined(__x86_64__) || NTL_BITS_PER_LONG != 64)
+#ifdef NTL_HAVE_ALIGNED_ARRAY
 
-// DIRT: for now, we only attempt to implement this function properly when
-// we really need it, which is for AVX support.
-// The source file CheckAVX.c checks for these same conditions.
-// We still define it, to simplify the overall code structure.
-
-static inline
-char *_ntl_make_aligned(char *p, long align)
-{
-   return p;
-}
-
-#else
-
-// DIRT: in the limited range of platforms for which we attempt to
-// implement it, this should work fine.
-
-static inline
+inline
 char *_ntl_make_aligned(char *p, long align)
 {
    unsigned long r =  (unsigned long) (((NTL_UPTRINT_T) (p)) % ((NTL_UPTRINT_T) (align)));
    return p + ((((unsigned long) (align)) - r) % ((unsigned long) (align)));
 }
 
-#define NTL_HAVE_ALIGNED_ARRAY
+#else
+
+
+inline
+char *_ntl_make_aligned(char *p, long align)
+{
+   return p;
+}
+
 
 #endif
 
@@ -469,12 +510,122 @@ char *_ntl_make_aligned(char *p, long align)
 
 #define NTL_AVX_LOCAL_ARRAY(x, type, n) NTL_ALIGNED_LOCAL_ARRAY(NTL_AVX_BYTE_ALIGN, x, type, n)
 
-#define NTL_DEFAULT_ALIGN (64)
+#define NTL_AVX512_BYTE_ALIGN (64)
+
+#define NTL_AVX512_LOCAL_ARRAY(x, type, n) NTL_ALIGNED_LOCAL_ARRAY(NTL_AVX512_BYTE_ALIGN, x, type, n)
+
+
+#define NTL_DEFAULT_ALIGN (128)
 // this should be big enough to satisfy any SIMD instructions,
 // and it should also be as big as a cache line
+// x86 has cache line size of 64, while Aarch64 has cache line size of 128
+// The cache line size requirement is not critical for correctness,
+// but can lead to better memory performance
 
 
-   
+
+#ifdef NTL_HAVE_BUILTIN_CLZL
+
+inline long 
+_ntl_count_bits(unsigned long x)
+{
+   return x ? (NTL_BITS_PER_LONG - __builtin_clzl(x)) : 0;
+}
+
+#else
+
+inline long 
+_ntl_count_bits(unsigned long x)
+{
+   if (!x) return 0;
+
+   long res = NTL_BITS_PER_LONG;
+   while (x < (1UL << (NTL_BITS_PER_LONG-1))) {
+      x <<= 1;
+      res--;
+   }
+
+   return res;
+}
 
 #endif
 
+
+
+
+#if (!defined(NTL_CLEAN_INT) && NTL_ARITH_RIGHT_SHIFT && (NTL_BITS_PER_LONG == (1 << (NTL_NUMBITS_BPL-1))))
+
+
+
+inline void
+_ntl_bpl_divrem(long a, long& q, long& r)
+{
+   q = a >> (NTL_NUMBITS_BPL-1);
+   r = a & (NTL_BITS_PER_LONG-1);
+}
+
+#else
+
+inline void
+_ntl_bpl_divrem(long a, long& q, long& r)
+{
+   q = a / NTL_BITS_PER_LONG;
+   r = a % NTL_BITS_PER_LONG;
+   if (r < 0) {
+      q--;
+      r += NTL_BITS_PER_LONG;
+   }
+}
+
+#endif
+
+inline void
+_ntl_bpl_divrem(unsigned long a, long& q, long& r)
+{
+   q = a / NTL_BITS_PER_LONG;
+   r = a % NTL_BITS_PER_LONG;
+}
+
+
+// vectors are grown by a factor of 1.5
+inline long _ntl_vec_grow(long n)
+{ return n + n/2; }
+
+
+template <class T>
+struct _ntl_is_char_pointer
+{
+ enum {value = false};
+};
+
+template <>
+struct _ntl_is_char_pointer<char*>
+{
+ enum {value = true};
+};
+
+template <>
+struct _ntl_is_char_pointer<const char*>
+{
+ enum {value = true};
+};
+
+template <bool, typename T = void>
+struct _ntl_enable_if
+{};
+
+template <typename T>
+struct _ntl_enable_if<true, T> {
+  typedef T type;
+};
+
+
+// returns x, disabling constant folding
+int _ntl_nofold(int x); 
+long _ntl_nofold(long x); 
+double _ntl_nofold(double x); 
+
+
+
+
+#endif
